package client;

import java.util.concurrent.atomic.AtomicBoolean;

import org.lwjgl.opengl.Display;

import client.game.Application;

/**
 * The application is rendered and updated ~30 times per second by this thread.
 *
 */
@SuppressWarnings("unused")
public class Processor implements Runnable {

	private final Application application;
	private final long TIMER_RESOLUTION = 1000000000;
	private final int MAX_FRAME_SKIPS = 10;
	private long totalUpdateCount = 0L;
	private float lastUpdateCount = 0L;
	private long secondsRunning = 0L;
	private final int requestedFPS = 30;
	private long lastFrameCount = 0L;
	private long averageFrameCount = 0L;
	private AtomicBoolean running = new AtomicBoolean(true);

	public Processor(final Application application) {
		this.application = application;
	}

	@Override
	public void run() {
		long startTime = 0L;
		long endTime = 0L;
		long timeDiff = 0L;
		long sleepTime = 0L;
		long overSleepTime = 0L;
		long excess = 0L;
		int frameSkips = 0;
		int frameCount = 0;
		long period = TIMER_RESOLUTION / requestedFPS;
		long totalFrameCount = 0L;
		float currentUPS = 0;
		float delta = 0;
		long previousSampleTime = System.nanoTime();
		while (running.get() && !Display.isCloseRequested()) {
			startTime = System.nanoTime();
			currentUPS = totalUpdateCount / (float) (secondsRunning == 0 ? 1 : secondsRunning);
			delta = requestedFPS / currentUPS;
			cycle(delta);
			try { application.render((int) Math.floor(lastFrameCount)); } catch (Exception e) { }
			endTime = System.nanoTime();
			timeDiff = endTime - startTime;
			sleepTime = (period - timeDiff) - overSleepTime;
			if (sleepTime > 0) {
				try {
					Thread.sleep(sleepTime / (TIMER_RESOLUTION / 1000));
				} catch(final InterruptedException ignore) {
					Thread.currentThread().interrupt();
					break;
				}
				overSleepTime = (System.nanoTime() - endTime) - sleepTime;
			} else {
				excess -= sleepTime;
				overSleepTime = 0L;
			}
			frameSkips = 0;
			while ((excess > period) && (frameSkips < MAX_FRAME_SKIPS)) {
				excess -= period;
				if (Display.isActive()) {
					cycle(delta);
				}
				frameSkips++;
			}
			frameCount++;
			totalFrameCount++;
			totalUpdateCount += frameSkips + 1;
			if (System.nanoTime() - previousSampleTime > TIMER_RESOLUTION) {
				secondsRunning++;
				lastFrameCount = frameCount;
				lastUpdateCount = currentUPS;
				frameCount = 0;
				averageFrameCount = totalFrameCount / secondsRunning;
				previousSampleTime =  System.nanoTime();
			}
		}
		try {
			Display.destroy();
		} catch (Exception ignore) {
		}
		Runtime.getRuntime().runFinalization();
		Runtime.getRuntime().exit(0);
	}

	private void cycle(final float delta) {
		if (delta <= 0) {
			return;
		}
		application.cycle(delta);
	}

	public void terminate() {
		running.set(false);
	}

}
