package client.game;

import com.threed.jpct.Config;
import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;

/**
 * A primitive local player, controlled by the user.
 */
@SuppressWarnings("serial")
public class LocalPlayer extends Object3D {

	public float rotationY = 0;
	private boolean positionUpdateFlag = false;
	private SimpleVector moveRes = new SimpleVector(0, 0, 0);
	public static final SimpleVector ellipsoid = new SimpleVector(2f, 2.5f, 2f);
	private final SimpleVector GRAVITY = new SimpleVector(0, .6f, 0);
	private final float DAMPING = 0.1f;
	private final float ROTATION_SPEED = 1.6f;
	private final float MOVEMENT_SPEED = .8f;
	private final float groundLevel;

	@SuppressWarnings("unused")
	private long lastPositionUpdate;

	private boolean jumping;
	private int jumpTick = 0;
	private long lastJumpRequest;

	public LocalPlayer(final World world, final Object3D model, final float groundLevel) {
		super(model);
		this.setTexture("cyan");
		this.compile(true);
		world.addObject(this);
		this.setCollisionMode(Object3D.COLLISION_CHECK_SELF);
		this.clearTranslation();
		this.clearRotation();
		this.rotateY(rotationY);
		this.rotateMesh();
		this.groundLevel = groundLevel;
	}

	private final float FRAME_DIFFERENCE = 0.075f; // The different in frames <animationIndex> between animations.
	private float animationIndex;
	private int animationId = 0;
	private long lastFrameUpdate = 0L;
	private long animationStartTime = 0L;

	public void animate() {
		if (lastFrameUpdate <= System.currentTimeMillis() - 10) {
			lastFrameUpdate = System.currentTimeMillis();
			long resetDelay = 500; // default
			if (animationId == 1) {
				// we need to end the movement animation if we're not actually moving.
				resetDelay = positionUpdateFlag ? resetDelay : 250;
			}
			if (animationStartTime <= System.currentTimeMillis() - resetDelay) {
				animationIndex = 1; // TODO : idle
			} else {
				animationIndex += 0.008f; // increment frame value
				if (animationIndex > (FRAME_DIFFERENCE * (animationId + 1) + FRAME_DIFFERENCE) + (animationId > 0 ? 0.1f  : -0.1f)) {
					this.animationIndex = (FRAME_DIFFERENCE * (animationId + 1)) + (animationId > 0 ? 0.1f  : -0.1f);
				}
			}
			this.animate(animationIndex, 1); // Let's animate the 3D Object
		}
	}

	public void cycle(World world) {
		if (positionUpdateFlag) {
			lastPositionUpdate = System.currentTimeMillis();
			
			// avoid high speeds
			if (moveRes.length() > MOVEMENT_SPEED) {
				moveRes.makeEqualLength(new SimpleVector(0, 0, MOVEMENT_SPEED));
			}

			// move the player
			this.translate(0, -0.025f, 0); // prevent sinking through terrain
			moveRes = this.checkForCollisionEllipsoid(moveRes, ellipsoid, 8);
			this.translate(moveRes);

			if (jumping && jumpTick < 5) {
				jumpTick++;
				//this.translate(0, -(GRAVITY.y + (ellipsoid.y / 4)), 0); // go up...
				this.translate(checkForCollisionSpherical(new SimpleVector(0, -(GRAVITY.y + (ellipsoid.y / 4)), 0), 3));
			}
			if (jumpTick > 4) {
				jumping = false;
				jumpTick = 0;
			}

			if (!jumping) {
				applyGravity();
			}

			// damping
			if (moveRes.length() > DAMPING) {
				moveRes.makeEqualLength(new SimpleVector(0, 0, DAMPING));
			} else {
				moveRes = new SimpleVector(0, 0, 0);
			}
		}
		if (!jumping) {
			positionUpdateFlag = false;
			applyGravity(); // apply a constant pull towards the ground
		}
	}

	private void applyGravity() {
		SimpleVector t = GRAVITY;
		t = this.checkForCollisionEllipsoid(t, ellipsoid, 1);
		this.translate(t);
	}

	public void moveForward() {
		SimpleVector t = this.getZAxis();
		t.scalarMul(MOVEMENT_SPEED);
		moveRes.add(t);
		positionUpdateFlag = true;
	}

	public void moveBackward() {
		SimpleVector t = this.getZAxis();
		t.scalarMul(-(MOVEMENT_SPEED / .25f));
		moveRes.add(t);
		positionUpdateFlag = true;
	}

	public void rotateLeft(boolean doubleTheSpeed) {
		rotationY += (float) Math.toRadians(-(doubleTheSpeed ? ROTATION_SPEED * 2 : ROTATION_SPEED));
		this.rotateY((float) Math.toRadians(-(doubleTheSpeed ? ROTATION_SPEED * 2 : ROTATION_SPEED)));
		positionUpdateFlag = true;
	}

	public void rotateRight(boolean doubleTheSpeed) {
		rotationY += (float) Math.toRadians((doubleTheSpeed ? ROTATION_SPEED * 2 : ROTATION_SPEED));
		this.rotateY((float) Math.toRadians((doubleTheSpeed ? ROTATION_SPEED * 2 : ROTATION_SPEED)));
		positionUpdateFlag = true;
	}

	public void goUp() {
		if (this.getTransformedCenter().y < -Config.farPlane / 2) {
			return;
		}
		this.translate(0, -(GRAVITY.y + ellipsoid.y), 0); // go up...
	}

	public void goDown() {
		if (this.getTransformedCenter().y > groundLevel) {
			return;
		}
		this.translate(0, (GRAVITY.y + ellipsoid.y), 0); // go down...
	}

	public void jump() {
		if (lastJumpRequest <= System.currentTimeMillis() - 800) {
			lastJumpRequest = System.currentTimeMillis();
			jumpTick = 0;
			jumping = true;
			positionUpdateFlag = true;
		}
	}

	public void printPosition() {
		System.out.println(this.getTransformedCenter().x + ", " + (this.getTransformedCenter().y - ellipsoid.y) + ", " + this.getTransformedCenter().z);
	}

	/**
	 * Makes this player perform an animation.
	 * 
	 * 0) swing
	 * 1) walk
	 * TODO more animations!!
	 */
	public void setAnimation(int animationId) {
		this.animationStartTime = System.currentTimeMillis();
		if (this.animationId != animationId) {
			this.animationId = animationId;
			this.animationIndex = (FRAME_DIFFERENCE * (animationId + 1)) + (animationId > 0 ? 0.1f  : -0.1f);
			System.out.println("performing animation #"+animationId+", starting frame="+animationIndex);
		}
	}

	public float getX() {
		return this.getTransformedCenter().x;
	}

	public float getZ() {
		return this.getTransformedCenter().z;
	}
	
	public boolean hasPositionUpdateFlag() {
		return positionUpdateFlag;
	}
	
}
