package client.game.block;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import shared.MathUtils;
import client.game.Cache;

import com.threed.jpct.Config;
import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;

/**
 * Manages all known blocks (sent from server), and updates which ones
 * the local client will render (every 1500ms)
 */
public class TileBlockManager {

	private final World world;
	private final Cache cache;
	private List<TileBlock> tileBlocks = new CopyOnWriteArrayList<TileBlock>();
	private long lastTileBlockVisibilityUpdate = 0L;
	private int tileBlocksInView = 0;

	public TileBlockManager(final World world, final Cache cache) {
		this.world = world;
		this.cache = cache;
	}

	/**
	 * This is slow and culls too much at the border of the viewport.
 	 * It's not meant to be used in a productive application.
 	 * 
 	 * UPDATE: Which is why we have this simple throttler.
	 */
	public void update(Object3D focusPoint) {
		if (lastTileBlockVisibilityUpdate <= System.currentTimeMillis() - 1500) {
			lastTileBlockVisibilityUpdate = System.currentTimeMillis();
			tileBlocksInView = 0;
			for (TileBlock tileBlock : tileBlocks) {
				/**
				 * First we need to know if the block is within range..
				 */
				if (MathUtils.isInRange(Config.farPlane, focusPoint.getTransformedCenter(), tileBlock.getTransformedCenter())) {
					/**
					 * Then we check if the block is inside of the player's camera view..
					 */
					if (isInCameraView(tileBlock)) {
						tileBlocksInView++;
					}
				} else if (tileBlock.getVisibility()) {
					tileBlock.setVisibility(false); // Disable the block, but keep it's data cached in memory.
				}
			}
		}
	}

	/**
	 * Determine which blocks are actually visible from the player's camera view.
	 * 
	 * @note This should be executed AFTER determining which blocks are within distance from
	 * the player.
	 */
	private boolean isInCameraView(final TileBlock tileBlock) {
		SimpleVector camera_position = world.getCamera().getPosition();
		SimpleVector camera_direction = world.getCamera().getDirection();
		SimpleVector pp = tileBlock.getTransformedCenter();
		SimpleVector to = pp.calcSub(camera_position);
		float len = to.length();
		if (len < (Config.farPlane + 25) && Math.abs(to.calcAngle(camera_direction)) < Math.PI / 3.5f) {
			tileBlock.setVisibility(true);
			return true;
		}
		return false;
	}

	public void registerTileBlock(boolean block, int uid, int type, float x, float y, float z) {
		TileBlock tileBlock = new TileBlock(cache.getTileBlockModel(block, type), block, uid, type, x, y, z);
		world.addObject(tileBlock);
		tileBlocks.add(tileBlock);
	}

	public void unregisterTileBlock(int uid) {
		for (TileBlock tileBlock : tileBlocks) {
			if (uid == tileBlock.getUid()) {
				world.removeObject(tileBlock);
				tileBlocks.remove(tileBlock);
			}
		}
	}

	public TileBlock getClosestTileBlock(SimpleVector position) {
		for (TileBlock tileBlock : tileBlocks) {
			if (MathUtils.isInRange(8, position, tileBlock.getTransformedCenter())) {
				return tileBlock;
			}
		}
		return null;
	}

	public int getCachedTileBlocks() {
		return tileBlocks.size();
	}

	public int getTileBlocksInView() {
		return tileBlocksInView;
	}

}
