package server.game.block;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.PrintWriter;
import java.io.StreamTokenizer;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicInteger;

import server.Server;
import server.game.Player;
import server.game.PlayerManager;

public class TileBlockManager {

	private static TileBlockManager singleton;;
	
	private AtomicInteger tileBlockUid = new AtomicInteger(0);
	private List<TileBlock> tileBlocks = new CopyOnWriteArrayList<TileBlock>();
	private long lastSave = System.currentTimeMillis();
	private boolean saveMapData;

	public TileBlockManager() {
		System.out.println("Tile / Block Manager initialized.");
		loadHostData();
	}

	public void update(final Server server) {
		if (saveMapData && lastSave <= System.currentTimeMillis() - 1 * (60000)) {
			lastSave = System.currentTimeMillis();
			try {
				PrintWriter pw = new PrintWriter(new FileOutputStream("./server_data/world.map"));
				for (TileBlock tileBlock : tileBlocks) {
					pw.println((tileBlock.isBlock() ? 1 : 0) + ", " + tileBlock.getType() + ", " + tileBlock.getX() + ", " + tileBlock.getY() + ", " + tileBlock.getZ());
				}
				pw.close();
				System.out.println("Saved " + tileBlocks.size() + " tiles / blocks.");
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	private void loadHostData() {
		try {
			File file = new File("./server_data/world.map");
			if (!file.exists()) {
				System.out.println("Could not load: " + file.getAbsolutePath());
			}
			int count = 0;
			StreamTokenizer tokenizer = new StreamTokenizer(new BufferedReader(new FileReader(file)));
			while (tokenizer.nextToken() != StreamTokenizer.TT_EOF) {
				boolean block = (int) tokenizer.nval == 1;
				tokenizer.nextToken();
				tokenizer.nextToken();
				int type = (int) tokenizer.nval;
				tokenizer.nextToken();
				tokenizer.nextToken();
				int x = (int) tokenizer.nval;
				tokenizer.nextToken();
				tokenizer.nextToken();
				int y = (int) tokenizer.nval;
				tokenizer.nextToken();
				tokenizer.nextToken();
				int z = (int) tokenizer.nval;
				registerTileBlock(block, type, x, y, z);
				count++;
			}
			System.out.println("Loaded " + count + " tile / blocks.");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Adds a block to the world.
	 */
	public void registerTileBlock(boolean block, int type, float x, float y, float z) {
		TileBlock tileBlock = new TileBlock(block, tileBlockUid.getAndIncrement(), type, x, y, z);
		for (TileBlock tb : tileBlocks) {
			if (tb.getX() == x && tb.getY() == y && tb.getZ() == z) {
				return; // square unavailable - cancel the request
			}
		}
		if (tileBlock != null) {
			for (Player p : PlayerManager.getInstance().getUsersOnline()) {
				if (p == null) {
					System.out.println("player is null!?");
				}
				p.getPacketDispatcher().registerTileBlock(tileBlock);
			}
			tileBlocks.add(tileBlock);
			saveMapData = true;
			System.out.println("[Server] Registered " + (block ? "block" : "tile") + " type="+type+", x="+x+", y="+y+", z="+z);
		}
	}

	/**
	 * Removes a block from the world.
	 */
	public void unregisterTileBlock(boolean block, int uid, float x, float y, float z) {
		TileBlock tileBlock = getTileBlockByUid(uid);
		if (tileBlock != null) {
			for (Player p : PlayerManager.getInstance().getUsersOnline()) {
				p.getPacketDispatcher().unregisterTileBlock(tileBlock);
			}
			tileBlocks.remove(tileBlock);
			saveMapData = true;
			System.out.println("[Server] Unregistered " + (block ? "block" : "tile") + " type="+uid+", x="+x+", y="+y+", z="+z);
		}
	}

	public TileBlock getTileBlockByUid(int uid) {
		for (TileBlock tb : tileBlocks) {
			if (tb.getUid() == uid) {
				return tb;
			}
		}
		return null;
	}

	public List<TileBlock> getTileBlocks() {
		return tileBlocks;
	}

	public static TileBlockManager getInstance() {
		if (singleton == null) {
			singleton = new TileBlockManager();
		}
		return singleton;
	}
	
}